<?php
/**
 * Custom Post Type for Wizard Forms and Taxonomies.
 */
class WordPress_Form_Wizard_Answers extends WordPress_Form_Wizard
{
    protected $plugin_name;
    protected $version;
    protected $prefix;

    /**
     * Constructor.
     *
     * @author Daniel Barenkamp
     *
     * @version 1.0.0
     *
     * @since   1.0.0
     * @link    http://www.welaunch.io
     *
     * @param string $plugin_name
     * @param string $version
     */
    public function __construct($plugin_name, $version)
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;

        $this->prefix = "wordpress_form_wizard_";
    }

    /**
     * Init.
     *
     * @author Daniel Barenkamp
     *
     * @version 1.0.0
     *
     * @since   1.0.0
     * @link    http://www.welaunch.io
     *
     * @return bool
     */
    public function init()
    {
        $this->register_form_wizard_post_type();

        add_action('wp_ajax_wordpress_form_wizard_add_answer', array($this, 'add_answer') );
        add_action('wp_ajax_wordpress_form_wizard_show_answer', array($this, 'show_answer') );
        add_action('wp_ajax_wordpress_form_wizard_hide_answer', array($this, 'hide_answer') );
        add_action('wp_ajax_wordpress_form_wizard_update_answer', array($this, 'update_answer') );
        add_action('wp_ajax_wordpress_form_wizard_sort_answers', array($this, 'sort_answers') );
        add_action('wp_ajax_wordpress_form_wizard_get_answers', array($this, 'get_answers') );
        add_action('wp_ajax_wordpress_form_wizard_search_posts', array($this, 'search_posts') );
    }

    /**
     * Save Points on Question Answered
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https:/welaunch.io
     * @return  [type]             [description]
     */
    public function show_answer()
    {
        $response = array(
            'status' => 0,
            'message' => '',
            'data' => array()
        );

        if(!current_user_can('edit_posts') ) {
            $response['message'] = esc_html__('No Access rights', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!defined('DOING_AJAX') || !DOING_AJAX) {
            $response['message'] = esc_html__('No AJAX call', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['answer']) || empty($_POST['answer'])) {
            $response['message'] = esc_html__('No answer.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        $answer = (int) $_POST['answer'];
        wp_publish_post( $answer );

        $response['status'] = 1;
        $response['message'] = esc_html__('answer published to Trash.', 'wordpress-form-wizard');
        $response['data'] = array(
            'id' => $answer,
        );

        echo json_encode($response);
        die();
    }

    /**
     * Save Points on Question Answered
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https:/welaunch.io
     * @return  [type]             [description]
     */
    public function hide_answer()
    {
        $response = array(
            'status' => 0,
            'message' => '',
            'data' => array()
        );

        if(!current_user_can('edit_posts') ) {
            $response['message'] = esc_html__('No Access rights', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!defined('DOING_AJAX') || !DOING_AJAX) {
            $response['message'] = esc_html__('No AJAX call', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['answer']) || empty($_POST['answer'])) {
            $response['message'] = esc_html__('No answer.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        $answer = (int) $_POST['answer'];
        wp_trash_post( $answer );

        $response['status'] = 1;
        $response['message'] = esc_html__('answer moved to Trash.', 'wordpress-form-wizard');
        $response['data'] = array(
            'id' => $answer,
        );

        echo json_encode($response);
        die();
    }

    public function search_posts()
    {
       $response = array(
            'status' => 0,
            'message' => '',
            'data' => array()
        );

        if (!defined('DOING_AJAX') || !DOING_AJAX) {
            $response['message'] = esc_html__('Not AJAX', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        $postId = intval($_POST['postId']);
        $search = esc_attr($_POST['search']);
        if(empty($postId) || empty($search)) {
            $response['message'] = esc_html__('Post ID or search missing', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        $postTypes = 'any';

        $question = get_post($postId);
        if(!$question) {
            $response['message'] = esc_html__('Question not found.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }
        $formId = $question->post_parent;
        $formPostTypes = get_post_meta($formId, $this->prefix . 'post_types', true);
        if(!empty($formPostTypes)) {
            $postTypes = $formPostTypes;
        }

        $queryArgs = array(
            'post_type' => $postTypes,
            'numberposts' => 10,
            's' => $search,
            'post_status' => 'any',
        );
        $posts = get_posts($queryArgs);

        if(!empty($posts)) {

            $response['items'] = array();
            $response['items'][] = array(
                'id' => 0,
                'text' => 'None',
            );

            foreach($posts as $post) {
                $response['items'][] = array(
                    'id' => $post->ID,
                    'text' => $post->post_title . ' (ID: ' . $post->ID . ')',
                );
            }
        }

        die(json_encode($response));

    }

    public function get_answers()
    {
       $response = array(
            'status' => 0,
            'message' => '',
            'data' => array()
        );

        if(!current_user_can('edit_posts') ) {
            $response['message'] = esc_html__('No Access rights', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!defined('DOING_AJAX') || !DOING_AJAX) {
            $response['message'] = esc_html__('No AJAX call', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['questionId']) || empty($_POST['questionId'])) {
            $response['message'] = esc_html__('Question id missing.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }
        $questionId = (int) $_POST['questionId'];

        $answers = get_post_meta($questionId, $this->prefix . 'answers', true);
        if(!empty($answers)) {
            $answers = get_posts(array(
                'post_type' => 'wizard_answer',
                'post__in' => $answers,
                'post_status' => 'any',
                'orderby' => 'post__in'
            ));

            $tmp = array();
            if(!empty($answers)) {
                foreach($answers as $answer) {
                    $tmp[$answer->ID] = $answer->post_title;
                }
            }
            $answers = $tmp;
        } else {
            $answers = array();
        }

        if (empty($answers)) {
            $response['message'] = esc_html__('No Answers found.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }


        $response['status'] = 1;
        $response['message'] = esc_html__('Answers found.', 'wordpress-form-wizard');
        $response['data'] = $answers;

        echo json_encode($response);
        die();

    }

    /**
     * Save Points on Question Answered
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https:/welaunch.io
     * @return  [type]             [description]
     */
    public function add_answer()
    {
        $response = array(
            'status' => 0,
            'message' => '',
            'data' => array()
        );

        if(!current_user_can('edit_posts') ) {
            $response['message'] = esc_html__('No Access rights', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!defined('DOING_AJAX') || !DOING_AJAX) {
            $response['message'] = esc_html__('No AJAX call', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['answer']) || empty($_POST['answer'])) {
            $response['message'] = esc_html__('No Question.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['postId']) || empty($_POST['postId'])) {
            $response['message'] = esc_html__('No Question.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        $answer = $_POST['answer'];
        $postId = $_POST['postId'];
        
        $answerId = wp_insert_post(array(
            'post_title' => $answer,
            'post_type' => 'wizard_answer',
            'post_parent' => $postId,
            'post_status' => 'publish',
        ));

        if (!$answerId) {
            $response['message'] = esc_html__('Answer could not be created.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        $answers = get_post_meta($postId, $this->prefix . 'answers', true);
        if(!empty($answers) && is_array($answers)) {
            $answers[] = $answerId;
        } else {
            $answers = array(
                $answerId
            );
        }

        update_post_meta($postId, $this->prefix . 'answers', $answers);

        $response['status'] = 1;
        $response['message'] = esc_html__('Answer created.', 'wordpress-form-wizard');
        $response['data'] = array(
            'id' => $answerId,
            'title' => $answer
        );

        echo json_encode($response);
        die();
    }

    /**
     * Save Points on Question Answered
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https:/welaunch.io
     * @return  [type]             [description]
     */
    public function sort_answers()
    {
        $response = array(
            'status' => 0,
            'message' => '',
            'data' => array()
        );

        if(!current_user_can('edit_posts') ) {
            $response['message'] = esc_html__('No Access rights', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!defined('DOING_AJAX') || !DOING_AJAX) {
            $response['message'] = esc_html__('No AJAX call', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['answerIds']) || empty($_POST['answerIds'])) {
            $response['message'] = esc_html__('Question Ids missing.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['postId']) || empty($_POST['postId'])) {
            $response['message'] = esc_html__('No Post Id.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        $answerIds = array_filter($_POST['answerIds'], 'is_numeric');
        $postId = $_POST['postId'];
        
        if (empty($answerIds)) {
            $response['message'] = esc_html__('Question Ids empty.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        update_post_meta($postId, $this->prefix . 'answers', $answerIds);
        $response['status'] = 1;
        $response['message'] = esc_html__('Question sorting updated.', 'wordpress-form-wizard');
        $response['data'] = array(
            'id' => $answerId
        );

        echo json_encode($response);
        die();
    }


    /**
     * Save Points on Question Answered
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https:/welaunch.io
     * @return  [type]             [description]
     */
    public function update_answer()
    {
        $response = array(
            'status' => 0,
            'message' => '',
            'data' => array()
        );

        if(!current_user_can('edit_posts') ) {
            $response['message'] = esc_html__('No Access rights', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!defined('DOING_AJAX') || !DOING_AJAX) {
            $response['message'] = esc_html__('No AJAX call', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['data']) || empty($_POST['data'])) {
            $response['message'] = esc_html__('Data Missing.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        $data = $_POST['data'];
        if (!isset($data['answer_id']) || empty($data['answer_id'])) {
            $response['message'] = esc_html__('Answer ID missing.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        $postId = $data['answer_id'];

        if(isset($data['answer_title']) || empty($data['answer_title'])) {
            // place the current post and $new_title into array
            $post_update = array(
                'ID'         => $postId,
                'post_title' => $data['answer_title']
            );

            wp_update_post( $post_update );
        }

        if(isset($data['answer_tooltip']) ) {
            update_post_meta($postId, $this->prefix . 'tooltip', $data['answer_tooltip']);
        }

        if(isset($data['answer_price']) ) {
            update_post_meta($postId, $this->prefix . 'price', $data['answer_price']);
        }

        if(isset($data['answer_description']) ) {
            update_post_meta($postId, $this->prefix . 'description', $data['answer_description']);
        }

        if(isset($data['answer_image']) ) {
            update_post_meta($postId, $this->prefix . 'image', $data['answer_image']);
        }

        if(isset($data['answer_taxonomies']) && !empty($data['answer_taxonomies']) ) {

            $tmp = array();
            foreach($data['answer_taxonomies'] as $taxonomiesData) {
                if(!isset($taxonomiesData['values']) || empty($taxonomiesData['name'])) {
                    continue;
                }
                $tmp[$taxonomiesData['name']] = array(
                    'operator' => $taxonomiesData['operator'],
                    'values' => $taxonomiesData['values']
                );
            }
            $data['answer_taxonomies'] = $tmp;

            update_post_meta($postId, $this->prefix . 'taxonomies', $data['answer_image']);
        }


        $response['status'] = 1;
        $response['message'] = esc_html__('Answer updated.', 'wordpress-form-wizard');
        $response['data'] = array(
            'id' => $questionId,
            'title' => $data['title'],
        );

        echo json_encode($response);
        die();
    }

    /**
     * Register Wizard Form Post Type.
     *
     * @author Daniel Barenkamp
     *
     * @version 1.0.0
     *
     * @since   1.0.0
     * @link    http://www.welaunch.io
     *
     * @return bool
     */
    public function register_form_wizard_post_type()
    {
        $singular = esc_html__('Wizard Answer', 'wordpress-form-wizard');
        $plural = esc_html__('Wizard Answers', 'wordpress-form-wizard');

        $labels = array(
            'name' => esc_html__('Wizard Answers', 'wordpress-form-wizard'),
            'all_items' => $plural,
            'singular_name' => $singular,
            'add_new' => sprintf(__('New %s', 'wordpress-form-wizard'), $singular),
            'add_new_item' => sprintf(__('Add New %s', 'wordpress-form-wizard'), $singular),
            'edit_item' => sprintf(__('Edit %s', 'wordpress-form-wizard'), $singular),
            'new_item' => sprintf(__('New %s', 'wordpress-form-wizard'), $singular),
            'view_item' => sprintf(__('View %s', 'wordpress-form-wizard'), $plural),
            'search_items' => sprintf(__('Search %s', 'wordpress-form-wizard'), $plural),
            'not_found' => sprintf(__('No %s found', 'wordpress-form-wizard'), $plural),
            'not_found_in_trash' => sprintf(__('No %s found in trash', 'wordpress-form-wizard'), $plural),
        );

        $args = array(
            'labels' => $labels,
            'public' => false,
            'exclude_from_search' => false,
            'show_ui' => false,
            'menu_position' => 57,
            'query_var' => 'wizard-answer',
            'supports' => array('title', 'author'),
            'menu_icon' => 'dashicons-welcome-learn-more',
        );

        register_post_type('wizard_answer', $args);

    }


    /**
     * Add custom ticket metaboxes
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @param   [type]                       $post_type [description]
     * @param   [type]                       $post      [description]
     */
    public function add_custom_metaboxes($post_type, $post)
    {
        // add_meta_box('wordpress-form_wizard-general', 'General', array($this, 'general'), 'form-wizard', 'normal', 'high');
        // add_meta_box('wordpress-form_wizard-answers', 'Answers', array($this, 'answers'), 'form-wizard', 'normal', 'high');
    }

    /**
     * Save Custom Metaboxes
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @param   [type]                       $post_id [description]
     * @param   [type]                       $post    [description]
     * @return  [type]                                [description]
     */
    public function save_custom_metaboxes($post_id, $post)
    {
        global $wordpress_form_wizard_options;

        if($post->post_type !== "form-wizard") {
            return false;
        }

        // Is the user allowed to edit the post or page?
        if (!current_user_can('edit_post', $post->ID)) {
            return $post->ID;
        }

        if ($post->post_type == 'revision') {
            return false;
        }

        if (!isset($_POST['wordpress_form_wizard_meta_nonce']) || !wp_verify_nonce($_POST['wordpress_form_wizard_meta_nonce'], basename(__FILE__))) {
            return false;
        }
        
        // Add values of $ticket_meta as custom fields
        foreach ($_POST as $postKey => $postData) {

            if(!str_contains( $postKey, 'wordpress_form_wizard_')) {
                continue;
            }

            update_post_meta($post->ID, $postKey, $postData);
        }
    }


}
